@@ -31,29 +31,47 @@
 #ifndef __GOOGLE_PROTOBUF_RUBY_PROTOBUF_H__
 #define __GOOGLE_PROTOBUF_RUBY_PROTOBUF_H__
 
-#include <ruby/ruby.h>
+// Ruby 3+ defines NDEBUG itself, see: https://bugs.ruby-lang.org/issues/18777
+#ifdef NDEBUG
+#include <ruby.h>
+#else
+#include <ruby.h>
+#undef NDEBUG
+#endif
+
+#include <ruby/version.h>
+
+#if RUBY_API_VERSION_CODE < 20700
+#error Protobuf requires Ruby >= 2.7
+#endif
+
+#include <assert.h>  // Must be included after the NDEBUG logic above.
+#include <ruby/encoding.h>
 #include <ruby/vm.h>
-#include <ruby/encoding.h>
 
-#include "ruby-upb.h"
 #include "defs.h"
+#include "ruby-upb.h"
 
 // These operate on a map field (i.e., a repeated field of submessages whose
 // submessage type is a map-entry msgdef).
-const upb_fielddef* map_field_key(const upb_fielddef* field);
-const upb_fielddef* map_field_value(const upb_fielddef* field);
+const upb_FieldDef* map_field_key(const upb_FieldDef* field);
+const upb_FieldDef* map_field_value(const upb_FieldDef* field);
 
 // -----------------------------------------------------------------------------
 // Arena
 // -----------------------------------------------------------------------------
 
-// A Ruby object that wraps an underlying upb_arena.  Any objects that are
+// A Ruby object that wraps an underlying upb_Arena.  Any objects that are
 // allocated from this arena should reference the Arena in rb_gc_mark(), to
 // ensure that the object's underlying memory outlives any Ruby object that can
 // reach it.
 
 VALUE Arena_new();
-upb_arena *Arena_get(VALUE arena);
+upb_Arena* Arena_get(VALUE arena);
+
+// Fuses this arena to another, throwing a Ruby exception if this is not
+// possible.
+void Arena_fuse(VALUE arena, upb_Arena* other);
 
 // Pins this Ruby object to the lifetime of this arena, so that as long as the
 // arena is alive this object will not be collected.
@@ -72,10 +90,9 @@ void Arena_Pin(VALUE arena, VALUE obj);
 // being collected (though in Ruby <2.7 is it effectively strong, due to
 // implementation limitations).
 
-// Adds an entry to the cache. The "arena" parameter must give the arena that
-// "key" was allocated from.  In Ruby <2.7.0, it will be used to remove the key
-// from the cache when the arena is destroyed.
-void ObjectCache_Add(const void* key, VALUE val);
+// Tries to add a new entry to the cache, returning the newly installed value or
+// the pre-existing entry.
+VALUE ObjectCache_TryAdd(const void* key, VALUE val);
 
 // Returns the cached object for this key, if any. Otherwise returns Qnil.
 VALUE ObjectCache_Get(const void* key);
@@ -89,10 +106,11 @@ typedef struct StringBuilder StringBuilder;
 
 StringBuilder* StringBuilder_New();
 void StringBuilder_Free(StringBuilder* b);
-void StringBuilder_Printf(StringBuilder* b, const char *fmt, ...);
+void StringBuilder_Printf(StringBuilder* b, const char* fmt, ...);
 VALUE StringBuilder_ToRubyString(StringBuilder* b);
 
-void StringBuilder_PrintMsgval(StringBuilder* b, upb_msgval val, TypeInfo info);
+void StringBuilder_PrintMsgval(StringBuilder* b, upb_MessageValue val,
+                               TypeInfo info);
 
 // -----------------------------------------------------------------------------
 // Utilities.
@@ -101,9 +119,13 @@ void StringBuilder_PrintMsgval(StringBuilder* b, upb_msgval val, TypeInfo info);
 extern VALUE cTypeError;
 
 #ifdef NDEBUG
-#define PBRUBY_ASSERT(expr) do {} while (false && (expr))
+#define PBRUBY_ASSERT(expr) \
+  do {                      \
+  } while (false && (expr))
 #else
-#define PBRUBY_ASSERT(expr) assert(expr)
+#define PBRUBY_ASSERT(expr) \
+  if (!(expr))              \
+  rb_bug("Assertion failed at %s:%d, expr: %s", __FILE__, __LINE__, #expr)
 #endif
 
 #define PBRUBY_MAX(x, y) (((x) > (y)) ? (x) : (y))
