@@ -331,10 +331,6 @@ bool Generator::Generate(const FileDescriptor* file,
   printer.Print("if _descriptor._USE_C_DESCRIPTORS == False:\n");
   printer_->Indent();
 
-  // We have to fix up the extensions after the message classes themselves,
-  // since they need to call static RegisterExtension() methods on these
-  // classes.
-  FixForeignFieldsInExtensions();
   // Descriptor options may have custom extensions. These custom options
   // can only be successfully parsed after we register corresponding
   // extensions. Therefore we parse all options again here to recognize
@@ -701,8 +697,9 @@ void Generator::PrintDescriptor(const Descriptor& message_descriptor) const {
   for (int i = 0; i < message_descriptor.extension_range_count(); ++i) {
     const Descriptor::ExtensionRange* range =
         message_descriptor.extension_range(i);
-    printer_->Print("($start$, $end$), ", "start", absl::StrCat(range->start),
-                    "end", absl::StrCat(range->end));
+    printer_->Print("($start$, $end$), ", "start",
+                    absl::StrCat(range->start_number()), "end",
+                    absl::StrCat(range->end_number()));
   }
   printer_->Print("],\n");
   printer_->Print("oneofs=[\n");
@@ -1006,49 +1003,6 @@ void Generator::FixForeignFieldsInDescriptors() const {
   printer_->Print("\n");
 }
 
-// We need to not only set any necessary message_type fields, but
-// also need to call RegisterExtension() on each message we're
-// extending.
-void Generator::FixForeignFieldsInExtensions() const {
-  // Top-level extensions.
-  for (int i = 0; i < file_->extension_count(); ++i) {
-    FixForeignFieldsInExtension(*file_->extension(i));
-  }
-  // Nested extensions.
-  for (int i = 0; i < file_->message_type_count(); ++i) {
-    FixForeignFieldsInNestedExtensions(*file_->message_type(i));
-  }
-  printer_->Print("\n");
-}
-
-void Generator::FixForeignFieldsInExtension(
-    const FieldDescriptor& extension_field) const {
-  ABSL_CHECK(extension_field.is_extension());
-
-  absl::flat_hash_map<absl::string_view, std::string> m;
-  // Confusingly, for FieldDescriptors that happen to be extensions,
-  // containing_type() means "extended type."
-  // On the other hand, extension_scope() will give us what we normally
-  // mean by containing_type().
-  m["extended_message_class"] =
-      ModuleLevelMessageName(*extension_field.containing_type());
-  m["field"] = FieldReferencingExpression(
-      extension_field.extension_scope(), extension_field, "extensions_by_name");
-  printer_->Print(m, "$extended_message_class$.RegisterExtension($field$)\n");
-}
-
-void Generator::FixForeignFieldsInNestedExtensions(
-    const Descriptor& descriptor) const {
-  // Recursively fix up extensions in all nested types.
-  for (int i = 0; i < descriptor.nested_type_count(); ++i) {
-    FixForeignFieldsInNestedExtensions(*descriptor.nested_type(i));
-  }
-  // Fix up extensions directly contained within this type.
-  for (int i = 0; i < descriptor.extension_count(); ++i) {
-    FixForeignFieldsInExtension(*descriptor.extension(i));
-  }
-}
-
 // Returns a Python expression that instantiates a Python EnumValueDescriptor
 // object for the given C++ descriptor.
 void Generator::PrintEnumValueDescriptor(
